==> v0.6.2.tar.gz: lifepex/src/pex.rb <==
require "date"
require_relative "./models/level"

def my_user_pexs cookies, date = nil
  params = {
    user_id: user_id_decoded(cookies),
  }
  params[:created_at] = date unless date.nil?
  LifePex::UserPex.where(params).all()
end

def get_user_date
  date = cookies["date"]
  if date && Date.respond_to?(date)
    Date.send date
  elsif date == "yesterday"
    Date.today - 1
  else
    Date.today
  end
end

class PexSystem < AuthSystem
  use AuthSystem

  get "/today", auth: [] do
    cookies["date"] = "today"
    redirect "/"
  end

  get "/yesterday", auth: [] do
    cookies["date"] = "yesterday"
    redirect "/"
  end

  get "/", auth: [] do
    user_pexs = my_user_pexs(cookies, get_user_date)
    user_pexs_amounts = user_pexs.group_by { |user_pex| user_pex[:pex_id] }
    pexs = current_user.pexs.to_a
    pex_by_models = pexs.map do |pex|
      {
        id: pex[:id],
        name: pex[:name],
        amount: pex[:amount],
        category: pex[:category],
        user_pexs: {
          amount: (user_pexs_amounts[pex[:id]] || []).length
        },
      }
    end
    slim :index, locals: { pex_by_models: pex_by_models }
  end

  post "/", auth: [] do
    user_id = user_id_decoded(cookies)
    pex_id = params["id"]
    if params["type"] == "+"
      LifePex::UserPex.insert(user_id: user_id, pex_id: pex_id, created_at: get_user_date)
    elsif params["type"] == "-"
      id = LifePex::UserPex.where(user_id: user_id, pex_id: pex_id).select(:id).first[:id]
      LifePex::UserPex.where(id: id).delete
    end
    redirect "/"
  end

  get "/pexs", auth: [] do
    slim :pex_form
  end

  get "/pexs/update", auth: [] do
    pex = LifePex::Pex.where(id: params["id"]).first
    slim :pex_form, locals: { pex: pex, submit_name: "update" }
  end

  post "/pexs", auth: [] do
    pex_id = if params["id"]
      LifePex::Pex.where(id: params["id"]).update(name: params['name'], category: params['category'], amount: params['amount'])
    else
      LifePex::Pex.insert(name: params['name'], category: params['category'], amount: params['amount'], user_id: current_user.id)
    end
    if pex_id
      redirect "/"
    else
      slim :register, locals: { flash: { danger: "Failed to add #{params['name']}" } }
    end
  end

  post "/pexs/delete", auth: [] do
    removed = LifePex::Pex.where(id: params['id']).delete
    links_removed = LifePex::UserPex.where(pex_id: params['id']).delete
    slim :pex_removed, locals: { flash: { success: "#{removed} model and #{links_removed} marks removed" } }
  end

  def compute_highchart_cumul(pex_data)
    total = 0
    pex_data.map do |v|
      total += v
    end
  end

  def get_recap_infos(cookies)
    user_pexs = my_user_pexs(cookies)
    pexs = LifePex::Pex.setup_user_pexs(user_pexs: user_pexs)
    total_xp = pexs.values.sum { |up| up[:total_by_date].values.sum }
    level = LifePex::Level.new total_xp
    all_dates = user_pexs.map(&:created_at).uniq
    all_category = pexs.values.map(&:category).uniq

    pex_table_by_date = all_dates.map { |date| [date, 0] }.to_h
    pexs.values.each do |pex|
      pex[:total_by_date].each do |date, total|
        # pex_table_by_date[date] ||= 0
        pex_table_by_date[date] += total
      end
    end

    pex_table_by_category_and_date = all_category.map do |category|
      [
        category,
        all_dates.map { |date| [date, 0] }.to_h,
      ]
    end.to_h

    pexs.values.each do |pex|
      pex[:total_by_date].each do |date, total|
        category = pex[:category]
        # pex_table_by_category_and_date[category] ||= {}
        # pex_table_by_category_and_date[category][date] ||= 0
        pex_table_by_category_and_date[category][date] += total
      end
    end

    highchart_total = {
      name: 'total',
      data: compute_highchart_cumul(pex_table_by_date.values),
    }
    highchart_tables = pex_table_by_category_and_date.map do |category, total_by_date|
      {
        name: category,
        data: compute_highchart_cumul(total_by_date.values),
      }
    end.unshift(highchart_total)

    {
      pex_tables: highchart_tables,
      level: level,
    }
  end

  get "/recap", auth: [], provides: 'html' do
    recap_infos = get_recap_infos(cookies)
    slim :recap, locals: recap_infos
  end

  get "/recap", auth: [], provides: 'json' do
    recap_infos = get_recap_infos(cookies)
    {
      pex_tables: recap_infos[:pex_tables],
      level: recap_infos[:level].to_h,
    }.to_json
  end
end

==> v0.7.0.tar.gz: lifepex/src/pex.rb <==
require "date"
require_relative "./models/level"

def my_user_pexs cookies, date = nil
  params = {
    user_id: user_id_decoded(cookies),
  }
  params[:created_at] = date unless date.nil?
  LifePex::UserPex.where(params).all()
end

def get_user_date
  date = cookies["date"]
  if date && Date.respond_to?(date)
    Date.send date
  elsif date == "yesterday"
    Date.today - 1
  else
    Date.today
  end
end

class PexSystem < AuthSystem
  use AuthSystem

  get "/today", auth: [] do
    cookies["date"] = "today"
    redirect "/"
  end

  get "/yesterday", auth: [] do
    cookies["date"] = "yesterday"
    redirect "/"
  end

  get "/", auth: [] do
    user_pexs = my_user_pexs(cookies, get_user_date)
    user_pexs_amounts = user_pexs.group_by { |user_pex| user_pex[:pex_id] }
    pexs = current_user.pexs.to_a
    pex_by_models = pexs.map do |pex|
      {
        id: pex[:id],
        name: pex[:name],
        amount: pex[:amount],
        category: pex[:category],
        user_pexs: {
          amount: (user_pexs_amounts[pex[:id]] || []).length
        },
      }
    end
    slim :index, locals: { pex_by_models: pex_by_models }
  end

  post "/", auth: [] do
    user_id = user_id_decoded(cookies)
    pex_id = params["id"]
    if params["type"] == "+"
      LifePex::UserPex.insert(user_id: user_id, pex_id: pex_id, created_at: get_user_date)
    elsif params["type"] == "-"
      id = LifePex::UserPex.where(user_id: user_id, pex_id: pex_id, created_at: get_user_date).select(:id).first[:id]
      LifePex::UserPex.where(id: id).delete
    end
    redirect "/"
  end

  get "/pexs", auth: [] do
    slim :pex_form
  end

  get "/pexs/update", auth: [] do
    pex = LifePex::Pex.where(id: params["id"]).first
    slim :pex_form, locals: { pex: pex, submit_name: "update" }
  end

  post "/pexs", auth: [] do
    pex_id = if params["id"]
      LifePex::Pex.where(id: params["id"]).update(name: params['name'], category: params['category'], amount: params['amount'])
    else
      LifePex::Pex.insert(name: params['name'], category: params['category'], amount: params['amount'], user_id: current_user.id)
    end
    if pex_id
      redirect "/"
    else
      slim :register, locals: { flash: { danger: "Failed to add #{params['name']}" } }
    end
  end

  post "/pexs/delete", auth: [] do
    removed = LifePex::Pex.where(id: params['id']).delete
    links_removed = LifePex::UserPex.where(pex_id: params['id']).delete
    slim :pex_removed, locals: { flash: { success: "#{removed} model and #{links_removed} marks removed" } }
  end

  def compute_highchart_cumul(pex_data)
    total = 0
    pex_data.map do |v|
      total += v
    end
  end

  def get_recap_infos(cookies)
    user_pexs = my_user_pexs(cookies)
    pexs = LifePex::Pex.setup_user_pexs(user_pexs: user_pexs)
    total_xp = pexs.values.sum { |up| up[:total_by_date].values.sum }
    level = LifePex::Level.new total_xp
    all_dates = user_pexs.map(&:created_at).uniq
    all_category = pexs.values.map(&:category).uniq

    pex_table_by_date = all_dates.map { |date| [date, 0] }.to_h
    pexs.values.each do |pex|
      pex[:total_by_date].each do |date, total|
        # pex_table_by_date[date] ||= 0
        pex_table_by_date[date] += total
      end
    end

    pex_table_by_category_and_date = all_category.map do |category|
      [
        category,
        all_dates.map { |date| [date, 0] }.to_h,
      ]
    end.to_h

    pexs.values.each do |pex|
      pex[:total_by_date].each do |date, total|
        category = pex[:category]
        # pex_table_by_category_and_date[category] ||= {}
        # pex_table_by_category_and_date[category][date] ||= 0
        pex_table_by_category_and_date[category][date] += total
      end
    end

    highchart_total = {
      name: 'total',
      data: compute_highchart_cumul(pex_table_by_date.values),
    }
    highchart_tables = pex_table_by_category_and_date.map do |category, total_by_date|
      {
        name: category,
        data: compute_highchart_cumul(total_by_date.values),
      }
    end.unshift(highchart_total)

    {
      pex_tables: highchart_tables,
      level: level,
    }
  end

  get "/recap", auth: [], provides: 'html' do
    recap_infos = get_recap_infos(cookies)
    slim :recap, locals: recap_infos
  end

  get "/recap", auth: [], provides: 'json' do
    recap_infos = get_recap_infos(cookies)
    {
      pex_tables: recap_infos[:pex_tables],
      level: recap_infos[:level].to_h,
    }.to_json
  end
end
